@@ -84,8 +84,13 @@ namespace GtkSharp.Generation {
 			if (!retval.Validate (log) || !base.Validate (log))
 				return false;
 
+			if (Name == String.Empty || CName == String.Empty) {
+				log.Warn ("Method has no name or cname.");
+				return false;
+			}
+
 			Parameters parms = Parameters;
-			is_get = ((((parms.IsAccessor && retval.IsVoid) || (parms.Count == 0 && !retval.IsVoid)) || (parms.Count == 0 && !retval.IsVoid)) && HasGetterName);
+			is_get = ((parms.IsAccessor && retval.IsVoid) || (parms.Count == 0 && !retval.IsVoid)) && HasGetterName;
 			is_set = ((parms.IsAccessor || (parms.VisibleCount == 1 && retval.IsVoid)) && HasSetterName);
 
 			call = "(" + (IsStatic ? "" : container_type.CallByName () + (parms.Count > 0 ? ", " : "")) + Body.GetCallString (is_set) + ")";
@@ -128,6 +133,9 @@ namespace GtkSharp.Generation {
 			else if (Modifiers == "new " || (dup != null && ((dup.Signature != null && Signature != null && dup.Signature.ToString() == Signature.ToString()) || (dup.Signature == null && Signature == null))))
 				sw.Write("new ");
 
+			if (Name.StartsWith (container_type.Name))
+				Name = Name.Substring (container_type.Name.Length);
+
 			if (is_get || is_set) {
 				if (retval.IsVoid)
 					sw.Write (Parameters.AccessorReturnType);
@@ -207,6 +215,14 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public void GenerateOverloads (StreamWriter sw)
+		{
+			sw.WriteLine ();
+			sw.WriteLine ("\t\tpublic " + retval.CSType + " " + Name + "(" + (Signature != null ? Signature.WithoutOptional () : "") + ") {");
+			sw.WriteLine ("\t\t\t{0}{1} ({2});", !retval.IsVoid ? "return " : String.Empty, Name, Signature.CallWithoutOptionals ());
+			sw.WriteLine ("\t\t}");
+		}
+
 		public void Generate (GenerationInfo gen_info, ClassBase implementor)
 		{
 			Method comp = null;
@@ -266,6 +282,9 @@ namespace GtkSharp.Generation {
 			}
 			else
 				gen_info.Writer.WriteLine();
+
+			if (Parameters.HasOptional && !(is_get || is_set))
+				GenerateOverloads (gen_info.Writer);
 			
 			gen_info.Writer.WriteLine();
 
