@@ -1,8 +1,11 @@
 // GtkSharp.Generation.StructBase.cs - The Structure/Boxed Base Class.
 //
-// Author: Mike Kestner <mkestner@speakeasy.net>
+// Authors:
+//   Mike Kestner <mkestner@speakeasy.net>
+//   Stephan Sundermann <stephansundermann@gmail.com>
 //
 // Copyright (c) 2001-2003 Mike Kestner
+// Copyright (c) 2013 Stephan Sundermann
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of version 2 of the GNU General Public
@@ -107,48 +110,68 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public virtual bool Union {
+			get {
+				return false;
+			}
+		}
+
 		protected void GenEqualsAndHash (StreamWriter sw)
 		{
 			int bitfields = 0;
 			bool need_field = true;
-			StringBuilder sb = new StringBuilder ();
+			StringBuilder hashcode = new StringBuilder ();
+			StringBuilder equals = new StringBuilder ();
 
-			sw.WriteLine ("\t\tpublic bool Equals ({0} other)", Name);
-			sw.WriteLine ("\t\t{");
+			hashcode.Append ("this.GetType ().FullName.GetHashCode ()");
+			equals.Append ("true");
 
 			foreach (StructField field in fields) {
+				if (field.IsPadding)
+					continue;
 				if (field.IsBitfield) {
 					if (need_field) {
-						sw.WriteLine ("\t\tif (!_bitfield{0}.Equals (other._bitfield{0})) return false;", bitfields);
-						if (sb.Length > 0)
-							sb.Append (" ^ ");
-						sb.Append ("_bitfield");
-						sb.Append (bitfields++);
-						sb.Append (".GetHashCode ()");
+						equals.Append (" && _bitfield");
+						equals.Append (bitfields);
+						equals.Append (".Equals (other._bitfield");
+						equals.Append (bitfields);
+						equals.Append (")");
+						hashcode.Append (" ^ ");
+						hashcode.Append ("_bitfield");
+						hashcode.Append (bitfields++);
+						hashcode.Append (".GetHashCode ()");
 						need_field = false;
 					}
 				} else {
 					need_field = true;
-					sw.WriteLine ("\t\t\tif (!{0}.Equals (other.{0})) return false;", field.EqualityName);
-					if (sb.Length > 0)
-						sb.Append (" ^ ");
-					sb.Append (field.EqualityName);
-					sb.Append (".GetHashCode ()");
+					equals.Append (" && ");
+					equals.Append (field.EqualityName);
+					equals.Append (".Equals (other.");
+					equals.Append (field.EqualityName);
+					equals.Append (")");
+					hashcode.Append (" ^ ");
+					hashcode.Append (field.EqualityName);
+					hashcode.Append (".GetHashCode ()");
 				}
 			}
-			sw.WriteLine ("\t\t\treturn true;");
-			sw.WriteLine ("\t\t}");
-			sw.WriteLine ();
+
+			if (!Elem.GetAttributeAsBoolean ("noequals")) {
+				sw.WriteLine ("\t\tpublic bool Equals ({0} other)", Name);
+				sw.WriteLine ("\t\t{");
+				sw.WriteLine ("\t\t\treturn {0};", equals.ToString ());
+				sw.WriteLine ("\t\t}");
+				sw.WriteLine ();
+			}
 			sw.WriteLine ("\t\tpublic override bool Equals (object other)");
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\treturn other is {0} && Equals (({0}) other);", Name);
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
-			if (Elem.GetAttribute ("nohash") == "true")
+			if (Elem.GetAttributeAsBoolean ("nohash"))
 				return;
 			sw.WriteLine ("\t\tpublic override int GetHashCode ()");
 			sw.WriteLine ("\t\t{");
-			sw.WriteLine ("\t\t\treturn {0};", sb.ToString ());
+			sw.WriteLine ("\t\t\treturn {0};", hashcode.ToString ());
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
 
@@ -158,12 +181,13 @@ namespace GtkSharp.Generation {
 		{
 			int bitfields = 0;
 			bool need_field = true;
+			StreamWriter sw = gen_info.Writer;
 
 			foreach (StructField field in fields) {
+				if (Union)
+					sw.WriteLine ("\t\t[FieldOffset(0)]");
 				if (field.IsBitfield) {
 					if (need_field) {
-						StreamWriter sw = gen_info.Writer;
-
 						sw.WriteLine ("\t\tprivate uint _bitfield{0};\n", bitfields++);
 						need_field = false;
 					}
@@ -207,7 +231,10 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("#region Autogenerated code");
 			if (IsDeprecated)
 				sw.WriteLine ("\t[Obsolete]");
-			sw.WriteLine ("\t[StructLayout(LayoutKind.Sequential)]");
+			if (Union)
+				sw.WriteLine ("\t[StructLayout(LayoutKind.Explicit)]");
+			else
+				sw.WriteLine ("\t[StructLayout(LayoutKind.Sequential)]");
 			string access = IsInternal ? "internal" : "public";
 			sw.WriteLine ("\t" + access + " partial struct {0} : IEquatable<{0}> {{", Name);
 			sw.WriteLine ();
