@@ -26,10 +26,10 @@ namespace GLib {
 		[DllImport ("libgio-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
 		static extern IntPtr g_app_info_get_all();
 
-		public static GLib.AppInfo[] GetAll() { 
+		public static GLib.IAppInfo[] GetAll () {
 			IntPtr raw_ret = g_app_info_get_all();
-			GLib.AppInfo[] ret = (GLib.AppInfo[]) GLib.Marshaller.ListPtrToArray (raw_ret, typeof(GLib.List), true, false, typeof(GLib.AppInfo));
+			GLib.IAppInfo[] ret = (GLib.IAppInfo[]) GLib.Marshaller.ListPtrToArray (raw_ret, typeof (GLib.List), true, false, typeof (GLib.IAppInfo));
 			return ret;
 		}
 	}
-}
\ No newline at end of file
+}
