@@ -190,10 +190,7 @@ namespace GLib {
 			return fullname + "Adapter";
 		}
 
-		[DllImport ("libglib-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
-		static extern void g_free (IntPtr item);
-
-		[DllImport ("libgobject-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
+		[DllImport (Global.GObjectNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern void g_object_unref (IntPtr item);
 
 		public void Empty ()
@@ -205,7 +202,7 @@ namespace GLib {
 					} else if (typeof (GLib.IWrapper).IsAssignableFrom (element_type)) {
 						g_object_unref (NthData (i));
 					} else {
-						g_free (NthData (i));
+						Marshaller.Free (NthData (i));
 					}
 				}
 			}
